#include "AFPMaster.h"

AFPMaster::AFPMaster(unsigned long baud, int txPin) {
  _baud = baud;
  _txPin = txPin;
  _ready = false;
}

void AFPMaster::begin() {
  AfpSerial.begin(_baud);
  _ready = true;
}

void AFPMaster::sendSync() {
  // from https://github.com/gandrewstone/LIN
  AfpSerial.end();
  pinMode(_txPin, OUTPUT);
  digitalWrite(_txPin, LOW);

  // use the slowest available BAUD
  unsigned long int brkbit = (1000000UL/(AFP_BAUD_MIN));
  unsigned long int brkbegin = brkbit*13;
  unsigned long int brkend = (1000000UL/(_baud));
  
  // delayMicroseconds unreliable above 16383 see arduino man pages
  if (brkbegin > 16383) (brkbegin/1000);
  else delayMicroseconds(brkbegin);

  // BREAK delimiter
  digitalWrite(_txPin, HIGH);  

  if (brkend > 16383) delay(brkend/1000);  
  else delayMicroseconds(brkend);

  AfpSerial.begin(_baud);
  AfpSerial.write('U');
}

int AFPMaster::sendRequest(char id, const char* msg) {
  if (!_ready)
    return AFP_ERROR;

  sendSync();
  AfpSerial.write(id);
  AfpSerial.print(msg);
  AfpSerial.write('!');

  if (!AfpSerial.find('{'))
    return AFP_ERROR;
  
  String txt = AfpSerial.readStringUntil('\n');

  // mininmal reply is {}\n
  int len = txt.length();
  if (len < 1 || len > AFP_BUFFER_MAX)
    return AFP_ERROR;

  // copy reply and recover lost char (not the best way)
  String txt_full = '{' + txt + '\n';
  strcpy(reply, txt_full.c_str());
  return AFP_OK;
}

void AFPMaster::sendBroadcast(const char* msg) {
  sendRequest('*', msg);
}

void AFPMaster::setBaud(unsigned long baud) {
  _baud = baud;
  AfpSerial.end();
  AfpSerial.begin(baud);
}
