#include "AFPSlave.h"

AFPSlave::AFPSlave(char id, int rxPin) {
  _id = id;
  _rxPin = rxPin;
  _ready = false;
}

void AFPSlave::begin() {
  AfpSerial.begin(AFP_BAUD_DEFAULT);
  _ready = true;
  // high impedance while we listen
  pinMode(_rxPin, INPUT);
}

char AFPSlave::getId() {
  return _id;
}

int AFPSlave::pollRequest() {
  if (!_ready || !AfpSerial.available()) {
    return AFP_REQ_NONE;
  }
  // new request, decode header
  String txt = AfpSerial.readStringUntil('!');

  // minimal request is *!
  if (txt.length() < 1 || txt.length() > AFP_BUFFER_MAX) {
    _ready = true;
    return AFP_REQ_NONE;
  }
  strcpy(request, txt.c_str());
  
  char request_id;
  int i = 0;
  int j = 0;
  // ignore sync field if it wasn't already ignored by hardware sync logic
  if (request[i] == 'U')
    i++;
  request_id = request[i];
  i++;
  for (; i<txt.length(); i++, j++) {
    if (request[i] == '!') {
      break;
    }
    request_msg[j] = request[i];
  }

  _ready = false;
  if (request_id == _id) {
    return AFP_REQ_THIS;
  } else if (request_id == '*') {
    _ready = true;
    return AFP_REQ_BROADCAST;
  } else {
    return AFP_REQ_OTHER;
  }
}

void AFPSlave::sendReply() {
  pinMode(_rxPin, OUTPUT);
  AfpSerial.write('{');
  char txt[AFP_FIELD_MAX];
  for (int i=0,j=0; i<_nField; i++) {
    if (!fieldEnabled[i])
      continue;
    if (i>0)
      AfpSerial.write(',');
    AfpSerial.write('"');
    AfpSerial.print(fieldNames[i]);
    AfpSerial.print("\":");
    AfpSerial.print(fieldValues[i]);
  }
  AfpSerial.write('}');
  AfpSerial.write('\n');

  // flush buffer (in case of hardware loopback)
  while (AfpSerial.available()) {
    AfpSerial.read();
  }
  pinMode(_rxPin, INPUT);
  _ready = true;
}

void AFPSlave::listenReply() {
  String txt = AfpSerial.readStringUntil('\n');
  _ready = true;
}

int AFPSlave::addField(const char* field_name) {
  int field_id = _nField;

  if (strlen(field_name) > AFP_FIELD_MAX) {
    return -1;
  }
  strcpy(fieldNames[field_id], field_name);
  sprintf(fieldValues[field_id], "\"?\"");
  fieldEnabled[field_id] = true;
  _nField++;
  return field_id;
}

void AFPSlave::enableField(int field_id) {
  fieldEnabled[field_id] = true;
}

void AFPSlave::disableField(int field_id) {
  fieldEnabled[field_id] = false;
}

void AFPSlave::disableAllFields() {
  for (int i=0; i<_nField; i++) {
    fieldEnabled[i] = false;
  }
}

void AFPSlave::updateField(int field_id, const char* txt, bool add_quotes) {
  int len = strlen(txt);
  if (len > AFP_FIELD_MAX-2) {
    return;
  }
  if (add_quotes) {
    fieldValues[field_id][0] = '"';
    strcpy(fieldValues[field_id]+1, txt);
    fieldValues[field_id][len+1] = '"';
    fieldValues[field_id][len+2] = '\0';
  } else {
    strcpy(fieldValues[field_id], txt);
  }
}

void AFPSlave::setBaud(unsigned long baud) {
  AfpSerial.end();
  AfpSerial.begin(baud);
  _baud = baud;
  _ready = true;
  pinMode(_rxPin, INPUT);
}
